@@ -24,7 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdlib.h>
 #include <X11/Xlib.h>
@@ -63,6 +63,11 @@ _gdk_init_input_core (GdkDisplay *display)
   private->display = display;
 }
 
+static void gdk_device_class_init (GdkDeviceClass *klass);
+static void gdk_device_dispose    (GObject        *object);
+
+static gpointer gdk_device_parent_class = NULL;
+
 GType
 gdk_device_get_type (void)
 {
@@ -71,26 +76,65 @@ gdk_device_get_type (void)
   if (!object_type)
     {
       static const GTypeInfo object_info =
-      {
-        sizeof (GdkDeviceClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) NULL,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (GdkDevicePrivate),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc) NULL,
-      };
+	{
+	  sizeof (GdkDeviceClass),
+	  (GBaseInitFunc) NULL,
+	  (GBaseFinalizeFunc) NULL,
+	  (GClassInitFunc) gdk_device_class_init,
+	  NULL,           /* class_finalize */
+	  NULL,           /* class_data */
+	  sizeof (GdkDevicePrivate),
+	  0,              /* n_preallocs */
+	  (GInstanceInitFunc) NULL,
+	};
       
       object_type = g_type_register_static (G_TYPE_OBJECT,
-                                            "GdkDevice",
+                                            g_intern_static_string ("GdkDevice"),
                                             &object_info, 0);
     }
   
   return object_type;
 }
 
+static void
+gdk_device_class_init (GdkDeviceClass *klass)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  gdk_device_parent_class = g_type_class_peek_parent (klass);
+
+  object_class->dispose  = gdk_device_dispose;
+}
+
+static void
+gdk_device_dispose (GObject *object)
+{
+  GdkDevicePrivate *gdkdev = (GdkDevicePrivate *) object;
+
+  if (gdkdev->display && !GDK_IS_CORE (gdkdev))
+    {
+#ifndef XINPUT_NONE
+      if (gdkdev->xdevice)
+        {
+          XCloseDevice (GDK_DISPLAY_XDISPLAY (gdkdev->display), gdkdev->xdevice);
+          gdkdev->xdevice = NULL;
+        }
+      g_free (gdkdev->axes);
+      gdkdev->axes = NULL;
+#endif /* !XINPUT_NONE */
+
+      g_free (gdkdev->info.name);
+      g_free (gdkdev->info.keys);
+      g_free (gdkdev->info.axes);
+
+      gdkdev->info.name = NULL;
+      gdkdev->info.keys = NULL;
+      gdkdev->info.axes = NULL;
+    }
+
+  G_OBJECT_CLASS (gdk_device_parent_class)->dispose (object);
+}
+
 /**
  * gdk_devices_list:
  *
@@ -313,29 +357,42 @@ gdk_input_set_extension_events (GdkWindow *window, gint mask,
   if (mode == GDK_EXTENSION_EVENTS_NONE)
     mask = 0;
 
+  iw = _gdk_input_window_find (window);
+
   if (mask != 0)
     {
-      iw = g_new(GdkInputWindow,1);
+      if (!iw)
+        {
+          iw = g_new(GdkInputWindow,1);
 
-      iw->window = window;
-      iw->mode = mode;
+          iw->window = window;
+          iw->mode = mode;
 
-      iw->obscuring = NULL;
-      iw->num_obscuring = 0;
-      iw->grabbed = FALSE;
+          iw->obscuring = NULL;
+          iw->num_obscuring = 0;
+          iw->grabbed = FALSE;
+
+          display_x11->input_windows = g_list_append(display_x11->input_windows,iw);
+        }
 
-      display_x11->input_windows = g_list_append(display_x11->input_windows,iw);
       window_private->extension_events = mask;
 
+#ifndef XINPUT_NONE
       /* Add enter window events to the event mask */
-      /* FIXME, this is not needed for XINPUT_NONE */
+      /* this is not needed for XINPUT_NONE */
       gdk_window_set_events (window,
 			     gdk_window_get_events (window) | 
 			     GDK_ENTER_NOTIFY_MASK);
+
+      /* we might not receive ConfigureNotify so get the root_relative_geometry
+       * now, just in case */
+      _gdk_input_get_root_relative_geometry (GDK_WINDOW_XDISPLAY (window),
+                                             GDK_WINDOW_XWINDOW (window),
+                                             &iw->root_x, &iw->root_y, NULL, NULL);
+#endif /* !XINPUT_NONE */
     }
   else
     {
-      iw = _gdk_input_window_find (window);
       if (iw)
 	{
 	  display_x11->input_windows = g_list_remove(display_x11->input_windows,iw);
@@ -390,14 +447,7 @@ _gdk_input_exit (void)
 	  if (!GDK_IS_CORE (gdkdev))
 	    {
 	      gdk_device_set_mode (&gdkdev->info, GDK_MODE_DISABLED);
-	      
-	      g_free(gdkdev->info.name);
-#ifndef XINPUT_NONE	  
-	      g_free(gdkdev->axes);
-#endif	  
-	      g_free(gdkdev->info.axes);
-	      g_free(gdkdev->info.keys);
-	      g_free(gdkdev);
+	      g_object_unref(gdkdev);
 	    }
 	}
       
