@@ -43,6 +43,8 @@ typedef struct _GdkScreenX11Class GdkScreenX11Class;
 #define GDK_IS_SCREEN_X11_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GDK_TYPE_SCREEN_X11))
 #define GDK_SCREEN_X11_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GDK_TYPE_SCREEN_X11, GdkScreenX11Class))
 
+typedef struct _GdkX11Monitor GdkX11Monitor;
+
 struct _GdkScreenX11
 {
   GdkScreen parent_instance;
@@ -55,6 +57,7 @@ struct _GdkScreenX11
   GdkWindow *root_window;
 
   /* Window manager */
+  long last_wmspec_check_time;
   Window wmspec_check_window;
   char *window_manager_name;
   /* TRUE if wmspec_check_window has changed since last
@@ -76,21 +79,36 @@ struct _GdkScreenX11
   gint navailable_types;
   GHashTable *visual_hash;
   GHashTable *colormap_hash;
+  GdkVisual *rgba_visual;
   
   /* Colormap Part */
   GdkColormap *default_colormap;
   GdkColormap *system_colormap;
-  
+  GdkColormap *rgba_colormap;
+
   /* X settings */
   XSettingsClient *xsettings_client;
   guint xsettings_in_init : 1;
   
-  /* Xinerama */
-  gint     num_monitors;
-  GdkRectangle *monitors;
+  /* Xinerama/RandR 1.2 */
+  gint		 n_monitors;
+  GdkX11Monitor	*monitors;
 
   /* Pango renderer object singleton */
   PangoRenderer *renderer;
+
+  /* Xft resources for the display, used for default values for
+   * the Xft/ XSETTINGS
+   */
+  gboolean xft_init;		/* Whether we've intialized these values yet */
+  gboolean xft_antialias;
+  gboolean xft_hinting;
+  gint xft_hintstyle;
+  gint xft_rgba;
+  gint xft_dpi;
+
+  GdkAtom cm_selection_atom;
+  gboolean is_composited;
 };
   
 struct _GdkScreenX11Class
@@ -107,6 +125,8 @@ GdkScreen * _gdk_x11_screen_new      (GdkDisplay *display,
 void _gdk_x11_screen_window_manager_changed (GdkScreen *screen);
 void _gdk_x11_screen_size_changed           (GdkScreen *screen,
 					     XEvent    *event);
+void _gdk_x11_screen_process_owner_change   (GdkScreen *screen,
+					     XEvent    *event);
 
 G_END_DECLS
 
